;*********************************************************************************************************
;**********			digitales Bandzhlwerk Version 2.1				**********
;*********************************************************************************************************
;**********  Projekt:	elektronisches Zhlwerk fr Kassettenrekorder				**********
;**********											**********
;**********  Aufgabe:	Anzeige von								**********
;**********		- absoluter Bandposition in Minuten und Sekunden			**********
;**********		  im Aufnahme-, Wiedergabe- und Spulmodus				**********
;**********		- Umdrehungszahl des Aufwickeltellers					**********
;**********		- beim Umspulen Ausgabe eines Schaltsignals bei Zhlwerkstellung 0000	**********
;**********		- Betriebsstunden im Aufnahme- oder Wiedergabemodus			**********
;*********************************************************************************************************
;**********  Hardware:	Controller ATMEGA 8 mit 4 MHz Quarz					**********
;**********		3 Lichtschranken A und B am Aufwickelteller sowie C am Kassettenfach,	**********
;**********		ein Taster, ein 4 oder 5-stelliges 7-Segment LED-Display,		**********
;**********		Spannungsversorgung aus 12 Volt~					**********
;*********************************************************************************************************
;**********		Idee, Hardware, Software:	Stefan Kneller				**********
;**********		mit Anregungen aus der EU-Patentanmeldung EP1246189A2			**********
;*********************************************************************************************************
;**********  Todo:	- am Bandanfang durch dickes Vorspannband verzgerte Synchronisation	**********
;**********		  Eine umdrehungs- oder radiusgefhrte Sperre ist nicht zielfhrend.	**********
;**********		  Wenn nach einer Speicherinitialisierung die Banddicke schnell in den 	**********
;**********		  verbotenen hohen Bereich luft bin ich im Vorspannband.		**********
;**********		  Nun knnte ich erneut eine Speicherinitialisierung anfordern, muss	**********
;**********		  dafr aber eine valide Banddicke gewonnen haben und vorhalten.	**********
;**********		- evtl. Startwert Speicherinitialisierung S aus 2 Umdrehungen gewinnen	**********
;**********		- evtl. Kennlinien Banddickenverlauf neu aufnehmen und Anpassen von:	**********
;**********			Schaltzeitpunkte und Strke fr Filtertiefen			**********
;**********		 	Syncfenster und Syncsperre kurz					**********
;**********											**********
;*********************************************************************************************************
;**********  Historie:	- die Umdrehungs- und Lagenzhler zhlen direkt die Weifelder ohne	**********
;**********		  den Umweg ber einen Felderzhler. (ist Umdrehungszahl und Lagenzahl)	**********
;**********		  Zur Anzeige werden diese Zhler durch Zebra dividiert.		**********
;**********		- die Sekundenberechnung erfolgt nicht mehr auf Basis von ganzen	**********
;**********		  Lagen sondern der feiner aufgelsten Lagenzahl.			**********
;**********		- die Speicherinitialisierung ist prziser und ein Bug entfernt		**********
;**********		- das Einlesen und Verarbeiten der Laufzeitwerte findet nur noch in	**********
;**********		  einem einzigen Speicher statt.					**********
;**********		- das Synchronkriterium "Ableitung der Banddicke" ist von der		**********
;**********		  Mittelwerttiefe abhngig.						**********
;**********		- Anzeigemodus "F SYNCFENSTER" etabliert				**********
;**********		- Blinken und Dimmen der Anzeige ist auf die Funktionen synchronisiert	**********
;**********		- Leuchtzeichen aus dem "Anzeige"-speicher werden mit dem		**********
;**********		  Anzeigemultiplex synchronisiert in den "Ausgabe"-speicher bergeben.	**********
;**********		- Lauftextausgabe auf ASCII-Tabelle umgebaut				**********
;**********		- Mglichkeit einer Nachsynchronisierung bei hoher Zeitdifferenz S zu T	**********
;**********		- Zeitdifferenz S zu T wird gleitend an Sollwert nachgefhrt		**********
;**********		- Lagen- und Sekundenwerte sind nach oben begrenzt			**********
;**********		- Code besser strukturiert und kommentiert				**********
;**********		- zu Debuggingzwecken detailierte Fehlercodes eingebaut			**********
;**********											**********
;*********************************************************************************************************
;**********  Beschreibung:									**********
;**********  											**********
;**********  Das Rechenwerk zhlt mittels zweier Lichtschranken und Schwarzweifelder auf der	**********
;**********  Aufwickeltrommel die Anzahl, Richtung und Geschwindigkeit der Umdrehungen des	**********
;**********  Wickeltellers.									**********
;**********  											**********
;**********  Im Display wird die abgelaufen Spielzeit "T" angezeigt.				**********
;**********  Ein kurzer Tastendruck bringt rotierend weitere Werte auf die Anzeige.		**********
;**********											**********
;**********  "T"  errechnete, abgelaufene Spielzeit						**********
;**********  "U"  Umdrehungen, rcksetzbar							**********
;**********  "h"  Kopfstunden, rcksetzbar							**********
;**********  "H"  Kopfstunden									**********
;**********  "_"  Lauftext bis 254 Zeichen Lnge						**********
;**********  ""  Banddicke auf 1 Nachkommastelle gerundet					**********
;**********  "b"  Banddicke, roh wie sie in die Filterung einluft				**********
;**********  ""  Banddicke, wie sie aus der Filterung kommt					**********
;**********  "A"  Differenz der gefilterten Banddicke zum letzten Feldverbeilauf in nm		**********
;**********	  (wenn Wert < 10,00 nm mit Vorzeichenanzeige)					**********
;**********  "I"  Filtertiefe des Filters U, Banddicke						**********
;**********  "F"  Umdrehungszhler Syncfenster							**********
;**********  "L"  Bandlagen, errechnet								**********
;**********  "S." abgelaufene Sekunden, errechnet						**********
;**********  "E"  Fehler zwischen errechneten Sekunden "S." und angezeigter Spielzeit "T"	**********
;**********       ein Minuszeichen bedeutet T < S.						**********
;**********  "J"  Dauer des letzten Segmentvorbeilaufs						**********
;**********  "n"  gemittelte Segmentlaufzeit ber die letzten (Mittelwerttiefe*)Umdrehungen	**********
;**********  "N"  gemittelte Segmentlaufzeit ber (Mittelwerttiefe*)Umdrehungen vor Delta_S	**********
;**********  "o"  vom Rechenwerk verwendeter Kernradius in mm					**********
;**********  "r"  Radius des Bandwickels in mm							**********
;**********  "d"  Debugwert, der an beliebiger Stelle des Programms bergebenen werden kann	**********
;**********       										**********
;**********  Die Anzeige "U" fr Umdrehungen gibt die Anzahl der Tellerumdrehungen an.		**********
;**********  Die Anzeige ist jederzeit mit einem langen Tastendruck rcksetzbar und steuert	**********
;**********  nur im Spulmodus bei Erreichen der Zhlwerkstellung "0000" den Nullstoppausgang an.**********
;**********  Wenn der Kassettenrekorder ber eine entsprechnede Funktion verfgt, kann diese	**********
;**********  hierduch angesteuert werden.							**********
;**********  Der berlauf des Umdrehungszhlwerkes erfolgt bei 2000 ("max_umdrehungen").	**********
;**********  											**********
;**********  Wenn der Rekorder in Wiedergabegeschwindigkeit luft wird aus der im		**********
;**********  Mittelwertspeicher S gegltteten Tellerdrehzahl und der Tellerdrehzahl von		**********
;**********  einigen Umdrehungen vorher ein Wert fr die Banddicke berechnet.			**********
;**********  Dieser Wert ist mit Schwankungen behaftet und wird durch das Filter U beruhigt.	**********
;**********  Unterschiedliche Filtertiefen werden whrend der Synchronisierphase angewendet.	**********
;**********  Nach der Synchronisierung wird mit zunehmendem Wickelradius die Filtertiefe	**********
;**********  reduziert.										**********
;**********  											**********
;**********  Anhand des Dickenwertes und der aktuellen Tellerdrehzahl wird die Zahl der		**********
;**********  aufgewickelten Bandlagen und daraus die abgelaufene Spielzeit errechnet.		**********
;**********  Die Anzeige "L" zeigt die errechneten Bandlagen, Wertebereich 0 bis 2000 Lagen.	**********
;**********  Die Anzeige "S." zeigt nach Synchronisation die errechnete abgelaufene Spielzeit	**********
;**********  in vollen Sekunden. Wertebereich 0 bis 4200.					**********
;**********  Dies geschieht zu Beginn der Synchronisierphase einmal, um schon eine geschtzte	**********
;**********  Zeit anzeigen zu knnen. Nur sichtbar bei Optionseinstellung "P2-A".		********** 
;**********  											**********
;**********  Wenn alle Werte innerhalb ihrer Grenzen und stabil sind, nicht aber vor Beginn	**********
;**********  des Synchronisierfensters wird das Flag "Synchron"	gesetzt und die quarzgefhrte	**********
;**********  Zeit im Display nur noch in kleinen Schritten an die errechnete Zeit nachgefhrt.	**********
;**********  (variable Nachfhrungsrate, vom Fehler "E" abhngig)				**********
;**********  Am Ende des Synchronisierfensters wird Synchronisierung erzwungen.			**********
;**********  Der synchrone Zustand wird durch Verlschen der "calc"-Anzeige oder im Modus	**********
;**********  "P2-A" durch den Dezimalpunkt auf der rechten Stelle der Anzeige angezeigt.	**********
;**********  Erreicht der Fehler zwischen Zeit im Display und errechneter Zeit im Verlauf	**********
;**********  mehr als 120 Sekunden und sind mehr als 100 Umdrehungen seit Start vergangen dann	**********
;**********  wird einmal pro Bandwechsel eine Nachsynchronisierung durchgefhrt. Dies wird	**********
;**********  durch blinkende Dezimalpunkte der Zeitanzeige signalisiert.			**********
;**********  											**********
;**********  Die Anzeige "T" zeigt die abgelaufen Spielzeit					**********
;**********  5-stellig in H:MM:SS oder 4-stellig in MM:SS					**********
;**********  											**********
;**********  Im Aufnahme- und Wiedergabebetrieb werden alle 8 Sekunden beide Kopfstundenzhler	**********
;**********  bedient. Ihre Tiefe von 24 Bit reicht fr 2^24/450 Sekunden = 37282 Stunden.	**********
;**********  Die Anzeige "H" zeigt volle Kopfstunden oder Kopfstunden/10 (Option P6-A) an.	**********
;**********  Der Zhler kann durch Ausschalten bei gedrckter Bedientaste zurckgesetzt werden.	**********
;**********  Die Anzeige "h" zeigt volle Kopfstunden oder Kopfstunden x 10 (Option P5-E) an.	**********
;**********  Der Zhler wird bei 1000 Stunden automatisch zurckgesetzt.			**********
;**********  Er kann zur Erinnerung an ein Intervall fr die Reinigung oder Entmagnetisierung	**********
;**********  dienen und wird mit einem langen Tastendruck manuell zurckgesetzt.		**********
;**********  											**********
;**********  Beim Ausschalten der Versorgungsspannung werden die Werte fr Zhlwerksstand,	**********
;**********  Banddicke, Kopfstunden, Anzeigebetriebsart und Optionen ins EEPROM gerettet.	**********
;**********  Nach dem Einschalten wird mit diesen Werten initialisiert.				**********
;**********  											**********
;**********  Bei gestopptem und spulendem Laufwerk kann die Zeitanzeige nicht mehr aus einer 	**********
;**********  Quarzzeit abgeleitet werden, sondern muss aus den Lagen "L" errechnet werden.	**********
;**********  Es ergibt immer einen Sprung in der Zeitanzeige, wenn das Laufwerk gestoppt	**********
;**********  wird und "T" und "S" unterschiedlichen Inhalt haben.				**********
;**********  Um das zu verhindern wird im Stoppmoment die Lagenzahl auf den Wert gezwungen,	**********
;**********  der sich aus Quarzzeitanzeige und Banddicke ergibt.				**********
;**********  											**********
;**********  Das ffnen des Kassettenfaches setzt die Werte "T", "S.","L" und "E" zurck.	**********
;**********											**********
;**********  Einschalten des Gerts mit gedrckter Bedientaste fhrt in die Optionswahl.	**********
;**********  Anzeige: "Pn-s", n = Optionsnummer, s = Status Aus oder Ein			**********
;**********  Jetzt kann durch kurzes Drcken auf den Taster der Status der Option gewechselt	**********
;**********  werden. 										**********
;**********  Ein langer Tastendruck fhrt zur nchsten Option.					**********
;**********  Zum Speichern und Verlassen des Mens den Taster so lange gedrckt halten,		**********
;**********  bis kein Menpunkt mehr im Display erscheint oder Gert ausschalten.		**********
;**********  											**********
;**********  P1-A	kleines Anzeigekarussell, Anzahl der Werte whlbar			**********
;**********  P1-E	vollstndiges Anzeigekarussell						**********
;**********  											**********
;**********  P2-A	Anzeige/Taster normale Funktion, nach Sync. rechter Dezimalpunkt an	**********
;**********  P2-E	whrend initialer Messphase Anzeige "CALC", Taster gesperrt		**********
;**********											**********
;**********  P3-A	nach Einschalten ist Anzeigemodus Zeitanzeige "T"			**********
;**********  P3-E	nach Einschalten wird der letzte Anzeigemodus geladen			**********
;**********											**********
;**********  P4-A	Betriebsstundenanzeige "h" in vollen Stunden				**********
;**********  P4-E	Betriebsstundenanzeige "h" in Stunden mit einer Nachkommastelle		**********
;**********											**********
;**********  P5-A	Betriebsstundenanzeige "H" in Stunden / 10				**********
;**********  P5-E	Betriebsstundenanzeige "H" in vollen Stunden				**********
;**********											**********
;**********  P6-A	alle Berechnungen mit festem Wickelkernradius 11,15 mm			**********
;**********  P6-E	am Bandanfang kann ein kleinerer Kernradius erkannt und			**********
;**********		mit diesem gerechnet werden						**********
;**********											**********
;**********  P7-A	fhrende Nullen anzeigen						**********
;**********  P7-E	fhrende Nullen unterdrcken (gilt nie fr Zeitanzeige "T")		**********
;**********											**********
;**********  Cnnn	Die Helligkeit der Anzeige mit jedem kurzem Tastendruck erhhen.	**********
;**********  		Rcksetzen nach 100%. nnn = Helligkeit in Prozent.			**********
;**********  											**********
;**********  X nn	Die Anzahl der Anzeigemodi im kleinen Karussell mit jedem kurzen	**********
;**********  		Tastendruck erhhen. Rcksetzen nach Maximum. nn = Anzahl Anzeigemodi	**********
;**********  											**********
;**********  nnnn	Den Gesamtbetriebsstundenzhler "H" mit jedem kurzen Tastendruck	**********
;**********     	um 10 Stunden erhhen, nnnn = Anzeige Gesamtstunden			**********
;**********  											**********
;**********  Verschiedene Fehler werden zur Laufzeit erkannt.					**********
;**********  Bei einem Fehler blinkt "FE-N", N = Fehlercode					**********
;**********  											**********
;**********	- FE-1: mehrere fehlerhafte Bytes im EEPROM, nicht sicher rekonstruiert		**********
;**********	- FE-2: ein oder mehrere fehlerhafte Bytes in EEPROM Bank A repariert		**********
;**********	- FE-3: ein oder mehrere fehlerhafte Bytes in EEPROM Bank B repariert		**********
;**********	- FE-4: ein oder mehrere fehlerhafte Bytes in EEPROM Bank C repariert		**********
;**********  	- FE-5: falscher Sprung in die ISR Vektortabelle				**********
;**********	- FE-6: Programmcounter luft bis zum Flashende hoch				**********
;**********  	- FE-7: in Abschaltroutine "sichere ins EEPROM" gelaufen			**********
;**********  	- FE-8: Watchdog hat ausgelst							**********
;**********											**********
;**********  Bei mehreren gesetzten Fehlern wird mit einem kurzen Tastendruck rotierend der	**********
;**********  nchste Fehlercode angezeigt.							**********
;**********  Lschen aller Fehler durch einen langen Tastendruck oder Gert abschalten.		**********
;**********  											**********
;**********  Die Anwendung geht von einer konstanten Bandgeschwindigkeit von 4,75 cm/Sek aus.	**********
;**********  Die Bandgeschwindigkeit kann von der Anwendung nicht gemessen werden und		**********
;**********  abweichende Werte fhren zu einer ungenauen Anzeige.				**********
;**********  Andere Bedingungen die zu Anzeigefehlern fhren:					**********
;**********  (eine quantitative Analyse wurde nicht durchgefhrt)				**********
;**********  - 	wahrer Wickelkernradius auerhalb der Norm (ein kleinerer Kernradius wird	**********
;**********     bei aktiver adaptiver Kernradiusermittlung am Bandanfang erkannt)		**********
;**********  - 	Vorspannband hat stark abweichende Banddicke					**********
;**********  -  Beschrnkung auf die Abtastung nur eines Wickeltellers				**********
;**********  -  arithmetisch gemittelte Drehzahlimpulse entsprechen nicht dem wahren Mittel	**********
;**********     (Da die Drehzahlimpulse immer schwanken werden sie ber mehrere volle		**********
;**********     Umdrehungen (Mittelwerttiefe/zebra) arithmetisch gemittelt. Da die		**********
;**********     Drehzahlabnahme aber nicht linear erfolgt ergibt sich dadurch ein Fehler.	**********
;**********     Er ist umso kleiner je kleiner die Mittelwerttiefe ist.				**********
;**********  - 	unterschiedlich straff gewickelter Bandwickel (extrem kleiner Fehler)		**********
;**********  - 	Fehler durch Rundungen und Einschrnkung von Wertebereichen			**********
;**********  - 	Controllertakt ungleich 4.000 KHz						**********
;**********  											**********
;**********  Wenn der Umdrehungszhler vorwrts und rckwrts unterschiedlich zhlt oder	**********
;**********  hakelig luft ist eine mechanische/elektrische Justage der Lichtschranken ntig.	**********
;**********  Im Spulmodus mit etwas lockererem Bandwickel wird keine angepasste Banddicke fr	**********
;**********  die Berechnungen angewendet, da lockere Bandwickel nur einen sehr kleinen		**********
;**********  Fehler verursachen.								**********
;**********											**********
;**********  Die adaptive Kernradiusanwendung ergibt leider nicht nur bessere Genauigkeiten :-(	**********
;**********											**********
;**********  Diese Softwareversion ist fr Zebra = 4 nicht getestet.				**********
;**********  Retten ins EEPROM und Nullstopp funktionieren nicht mit angeschlossenem Dragon.	**********
;**********  Nach dem Einschalten der Lichtschranken-LEDs muss bis zum Auslesen eines stabilen	**********
;**********  Zustands eine Mindestzeit verstreichen.						**********
;**********  Diese ist bei jedem Schaltungsaufbau leicht unterschiedlich!			**********
;**********											**********
;*********************************************************************************************************
;***************  an beliebigen Kassetten ermittelte Werte					**********
;*********************************************************************************************************
;mit festem Filterindex 4	Wickelradius	Lagen	Dicke m	Dicke m	Differenz
;				Millimeter		errechnet	Rekorder	%	absolut
;TDK SA-X    C90		25,75		1172	12,58		12,34		-1,9	-0,24
;SONY UX-S   C90          	25,10		1166	12,09		12,06		-0,2	-0,03
;TDK MA-G    C60		24,10		820	15,97		15,90		-0,4	-0,07
;TUCAN FE    C90		25,95		1139	13,38		12,44		-7,0	-0,94
;FUJI KII    C90 o. Filz 	25,10		800	17,30		17,37		 0,4	 0,07
;FUJI DRII   C60 o. Filz 	25,25		799	17,85		17,44		-2,3	-0,41
;SONY EX-F   C60		23,75		814	15,66		15,36		-1,9	-0,30
;NONAME      C60		25,00		792	17,68		17,42		-1,9	-0,24
;*********************************************************************************************************
;***************  Formelsammlung								**********
;*********************************************************************************************************
;***************  Counter1 misst die Dauer jedes Wei-Schwarzfeld-Vorbeilaufs an Geber A	**********
;***************  Grundformel:									**********
;***************	 counts = r * 2 * Pi * CLK/(Vo * Zebra * Teiler)			**********
;***************	 counts = Zhlerstand Timer nach einem Wei-Schwarz-Feld-Wechsel	**********
;***************  	      r = Wickelradius in mm						**********
;***************	     Pi = 3,14159							**********
;***************	    CLK = 4.000.000 Hz							**********
;***************	     Vo = Bandgeschwindigkeit in mm/s			hier 47,5	**********
;***************          Zebra = Anzahl weier Felder auf dem Radumfang	hier 8		**********
;***************	 Teiler = Vorteiler 16-Bit Counter			hier 64		**********
;***************             Ro = Wickelkernradius in mm			hier 11,15	**********
;***************              d = Banddicke in mm				typ. 0,015	**********
;***************         T_Segm = Zhlerstand Counter1						**********
;***************   T_Segm_delta = Zhlerstand Counter1 alt					**********
;***************        delta_s = Umdrehungszahl zwischen den Zhlerstnden neu und alt		**********
;***************              T = abgelaufene Spielzeit in Sekunden				**********
;***************										**********
;***************  	  fr r = 11 mm ergeben sich 11367 counts				**********
;***************	  fr r = 28 mm ergeben sich 28935 counts				**********
;*************** Umdrehungszahl = Wickeltellerumdrehungen * 2 * Zebra				**********
;***************      Lagenzahl = Bandlagen * 2 * Zebra						**********
;***************     Dickenwert = Banddicke in m * 100						**********
;*********************************************************************************************************
;***************  Banddicke:									**********
;***************	d = (T_segm-T_segm_delta_v)*Vo*Zebra*Teiler1/2*Pi*Clk*delta_v		**********
;***************										**********
;*********************************************************************************************************
;***************  Bandlagen: (aus Segmentlaufzeit und Banddicke)				**********
;***************  Grundformel: counts = r * 2 * Pi * CLK/(Vo * zebra * Vorteiler1)		**********
;***************  Weiter gilt: Lagen = (Radius_aktuell - Ro)/Banddicke				**********
;***************  beide Gleichungen zusammgefhrt:						**********
;***************        Lagen = (counts_aktuell*Vo*Zebra*Teiler1/(2*Pi*CLK))-Ro)/Banddicke	**********
;***************										**********
;*********************************************************************************************************
;***************  Bandlagen: (aus abgelaufenen Sekunden und Banddicke)				**********
;***************  L = (Wurzel((Banddicke/2+Ro)^2+T*Banddicke*Vo/Pi)-Banddicke/2-Ro)/Banddicke	**********
;***************										**********
;*********************************************************************************************************
;***************  abgelaufenen Sekunden:							**********
;***************	T = (2 * PI * Lagen * (Ro + (Lagen + 1) * (Banddicke / 2))) / Vo	**********
;***************										**********
;*********************************************************************************************************
;***************  Wickelradius:									**********
;***************		r = counts * Vo * Zebra * Teiler1/(2 * Pi * CLK) oder		**********
;***************                r = Ro + Lagen * d						**********
;***************										**********
;*********************************************************************************************************
.include"m8def.inc"

.equ	Kommentarzeilen	= 7

;*****  T E S T  ******************
.org flashend-(8*Kommentarzeilen)	;
	rjmp	fehler_flashende	;zur Untersuchung auftretender Resets
;*****  E N D E  ******************

.org flashend-(8*Kommentarzeilen-1)	;ASCII Klartext fr das Flashende

.db	" Tapetimer V2.1 "
.db	"  fuer Atmega8  "
.db	"  FuseH = 0xCF  "
.db	"  FuseL = 0x3D  "
.db	"Letzte Aenderung"
.db	"   06.07.2020   "
.db	" Stefan Kneller "

;*********************************************************************************************************
;***************  Definition der Konstanten							**********
;***************  WB = Wertebereich								**********
;*********************************************************************************************************
.equ	Bandgeschw	= 47500		;(47625) Bandgeschwindigkeit in m/s
.equ	Zebra		= 8		;Anzahl Weifelder auf dem Wickeltellerumfang } WB: 4 oder 8
.equ	Mittelwerttiefe	= 8		;(8) Anzahl Umdrehungen fr die Mittelwertbildung S, WB: 2,4,8,16
					;aber immer grer/gleich Delta_S
.equ	Delta_S		= 8		;(8) Banddickenmessung anhand Wickelzunahme nach N Umdrehungen, WB: 2,3,4,5 bis 32
					;die RAM-Gre begrenzt die Kombinationsmglichkeiten
.equ	k_0		= zebra*2	;eine Umdrehung		 } oft
.equ	k_1		= zebra*(mittelwerttiefe+delta_s)	;} bentigte
.equ	k_2		= zebra*bandgeschw*1000			;} Konstanten
;.equ	k_3		= 25132741	;2 * Pi * clk		 } 

.equ	def_Karussell	= 3		;default Anzahl Modi im "kleinen Anzeigekarussell"	} WB: min_karussell bis 20
.equ	min_Karussell	= 2		;kleinste Anzahl Modi im "kleinen Anzeigekarussell"	} WB: 1 bis 20
.equ	max_Karussell	= 20		;Anzahl Modi im "groen  Anzeigekarussell"     		} WB: 1 bis 20
.equ	max_Menue	= 11		;Anzahl Menpunkte			       		} WB: 1 bis 15
.equ	Pos_U		= 2		;Nr. Anzeigemodus Umdrehungen U			}
.equ	Pos_h		= 3		;Nr. Anzeigemodus Stunden h, rcksetzbar	} WB: 1 bis max_Karussell
.equ	Pos_L		= 5		;Nr. Anzeigemodus Text				}     1 = Zeitanzeige
.equ	max_Stunden_r	= 1000		;Bei N Stunden wird der "kleine" Stundenzhler genullt	} WB: 1 bis 37282

.equ	def_Helligkeit	= 30		;default Displayhelligkeit in %, WB: min_helligkeit bis 100, Wichtung gem Tabelle
.equ	min_Helligkeit	= 5		;kleinste erlaubte Helligkeit
.equ	Dauer_Kennung	= 25		;Einblenddauer Kennbuchstabe in Frames			} WB: 0 bis 255
.equ	Scrollintervall	= 12		;Geschwindigkeit Lauftext in Frames/Zeichenwechsel	} WB: 0 bis 255

;.equ	min_Kern_init	= 11300		;Radiussperre fr Initialisierung ohne dickes Vorspannband
.equ	min_Kern	= 10790		;kleinster erlauber Kernradius bei adaptiver Kernradiusverwendung
.equ	def_Kern	= 11150		;Defaultradius des leeren Wickeltellers in m
					;4 in dieser Softwareversion nicht mehr getestet
.equ	min_Cue		= zebra*250/10	;(200)	empirisch ermittelt
.equ	max_Cue		= zebra*375	;(3000)	Fenster fr Bandgeschwindigkeit um als Spulen zu gelten
.equ	min_Play	= (def_kern*39269908)/(Bandgeschw*Zebra*115)	;10000	115%, K = 200*Pi*CLK/Vorteiler1
.equ	max_Play	= (def_kern*39269908)/(Bandgeschw*Zebra*40)	;30000	40%
					;wegen Streuung am Gert mssen o.a. Zahlen grozgig bemessen sein.
.equ	Laufwerk_dreht	= 24/zebra+2	;erlaubte Zeit in Frames ohne Gebersignal fr "Laufwerksbewegung vorhanden"
.equ	Dauer_Nullstopp	= 8		;Impulsdauer des Nullstoppsignals in Frames	WB: 1 bis 200

.equ	min_Dicke	= 8000		;kleinste erlaubte Banddicke	} 
.equ	max_Dicke	= 20000		;grte erlaubte Banddicke	} in nm
.equ	def_Dicke	= 12000		;Default Dickenwert		} 
.equ	max_Lagen	= 2000		;grte erlaubte Anzahl von Bandlagen
.equ	max_Umdrehung	= 2000		;berlauf des Umdrehungszhlwerkes
.equ	max_Sekunden	= 4200		;(70 Minuten) grtes erlaubtes Ergebnis Sekundenberechnung in Sekunden
.equ	Delta_U		= 40/mittelwerttiefe+1		;nderungsrate in nm/Segmentvorbeilauf darf
.equ	Delta_T		= 4		;(4) N Umdrehungen lang nicht berschritten werden fr Bedingung "Synchron"
					;WB: 1 bis (Syncpflicht-Syncsperre)
.equ	Syncsperre_kurz	= 8		;(8) N Umdrehungen lang nach Wiedegabekommando ohne Bercksichtigung
					;des Synchronstatus Zeitrechnung und -nachfhrung sperren		} WB:
.equ	Syncsperre	= 10		;(16) N Umdrehungen nach Messbeginn Synchronisierung sperren		} 1 bis
.equ	Syncpflicht	= 50		;(50) N Umdrehungen nach Messbeginn Synchronisierung erzwingen		} max. Syncfenster
.equ	max_Syncfenster	= 100		;bei N Umdrehungen stoppt das Syncfenster und ein Resync wird erlaubt	} WB: 1 bis 255
.equ	Delta_Resync	= 120		;(120) Differenz S. zu T in Sekunden bei der eine Nachsynchronisierung angefordert wird
.equ	Filterwechsel_1	= 16		;(24) N Umdrehungen nach Messbeginn von 1. auf 2. Filterindex umschalten} WB:
.equ	Filterwechsel_2	= 26		;(36) N Umdrehungen nach Messbeginn von 2. auf 3. Filterindex umschalten} 1 bis max. Syncfenster
.equ	Filterwechsel_3	= 16		;(16) bei Wickelradius > 16mm und Synchron 5. Filterindex nutzen
.equ	Filterwechsel_4	= 23		;(22) bei Wickelradius > 22mm und Synchron 6. Filterindex nutzen
.equ	Filterindex_1	= 5		;(4) zur schnellen Wertestabilisierung nach Anlauf		}
.equ	Filterindex_2	= 6		;(5) Zwischenwert						}
.equ	Filterindex_3	= 7		;(6) wirksam bis zur Synchronisierung				} WB:
.equ	Filterindex_4	= 11		;nach Synchronisierung						} N=1 bis N=16
.equ	Filterindex_5	= 10		;nach Synchronisierung und groem Wickelradius			} fr Filtertiefe 2^N 
.equ	Filterindex_6	= 9		;nach Synchronisierung und sehr groem Wickelradius		}
.equ	Prellen		= 2		;Zeitdauer in Frames die als max. Tasterprellen gelten	} WB: 1 bis 16
.equ	Feldersperre	= 2*k_0		;(2) als Rechnungsgrundlage gesperrte Umdrehungen nach Playbeginn } WB: 1 bis 2*zebra*(mittelwerttiefe+delta_s-1)
.equ	Kopfintervall	= 8		;Zeitintervall fr Kopfstundenzhlung in Sekunden	} WB: 1 bis 8
					;Vergrerung ber 8 erfordert wegen berlauf in der Routine "rechne_stunden_r" nderungen
.equ	Adr_EEPROM	= 32		;unterste genutzte Adresse im EEPROM } WB: 1 bis 81
					;ein "AA" dort zeigt ein initialisiertes EEPROM

.equ	Geraet		= 3		;fr die Zuordnung: Anzeigeart, Lauftext, Optionseinstellung, Betriebsstunden

.if (Geraet==1)				;1=TC650M
.equ	gem_Anode	= 0		;Anzeigeverdrahtung: 0 = gemeinsame Kathode / 1 = gemeinsame Anode
.equ	Anzeigestellen	= 4		;wieviele Stellen hat die LED-Anzeige? Wertebereich: 4 und 5
.equ	def_Option	= (1<<volle_modi)|(1<<stunden_zehntel)|(1<<stunden_volle)|(1<<nullunterdr)
					;|(1<<startmodus)|(1<<kern_variabel)|(1<<calcanzeige)
.equ	def_Stunden	= 507		;Default-Wert Kopfstunden
.equ	def_Stunden_r	= 32		;Default-Wert Kopfstunden, rcksetzbar
.endif

.if (Geraet==2)				;2=TC750 
.equ	gem_Anode	= 0		;Anzeigeverdrahtung: 0 = gemeinsame Kathode / 1 = gemeinsame Anode
.equ	Anzeigestellen	= 4		;wieviele Stellen hat die LED-Anzeige? Wertebereich: 4 und 5
.equ	def_Option	= (1<<volle_modi)|(1<<stunden_zehntel)|(1<<stunden_volle)|(1<<nullunterdr)|(1<<calcanzeige)
.equ	def_Stunden	= 93		;Default-Wert Kopfstunden
.equ	def_Stunden_r	= 3		;Default-Wert Kopfstunden, rcksetzbar
.endif

.if (Geraet==3)				;3=neutral
.equ	gem_Anode	= 1		;Anzeigeverdrahtung: 0 = gemeinsame Kathode / 1 = gemeinsame Anode
.equ	Anzeigestellen	= 5		;wieviele Stellen hat die LED-Anzeige? Wertebereich: 4 und 5
.equ	def_Option	= (1<<nullunterdr)|(1<<calcanzeige)
.equ	def_Stunden	= 0		;Default-Wert Kopfstunden
.equ	def_Stunden_r	= 0		;Default-Wert Kopfstunden, rcksetzbar
.endif

;*********************************************************************************************************
;***************  Pinbelegung									**********
;*********************************************************************************************************
;                           _______
;                        1 /       |28
; Reset            C6 o---|        |---o   C5 Ausgang Treiber Digit 5
; Segmentausgang   D0 o---|        |---o   C4 Ausgang Treiber Digit 4
; Segmentausgang   D1 o---|        |---o   C3 Ausgang Treiber Digit 3
; Segmentausgang   D2 o---|        |---o   C2 Ausgang Treiber Digit 2
; Segmentausgang   D3 o---|        |---o   C1 Ausgang Treiber Digit 1
; Segmentausgang   D4 o---| ATMEGA |---o   C0 Ausgang Lichtschranken-LEDs
;                 VCC o---|    8   |---o   GND
;                 GND o---|        |---o   AREF
;                XTAL o---|        |---o   AVCC
;                XTAL o---|        |---o   B5 Ausgang "Nullstopp" und "Debug-LED"
; Segmentausgang   D5 o---|        |---o   B4 Eingang "Versorgungsspannung vorhanden"
; Segmentausgang   D6 o---|        |---o   B3 Eingang Taster
; Segmentausgang   D7 o---|        |---o   B2 Gebereingang C
; Gebereingang A   B0 o---|________|---o   B1 Gebereingang B

;*********************************************************************************************************
;***************  Verdrahtung Port B								**********
;*********************************************************************************************************
;PORT B
;BIT    7         6         5         4         3         2         1         0
;      NC        NC      IR-LED    Spannung   Taster    Geber C   Geber B   Geber A

.equ	A_LED		= 5		;Schaltausgang LED		}low-aktiv
.equ	A_Stop		= 5		;Schaltausgang Nullstopp	}high-aktiv
.equ	E_Spannung	= 4		;Eingang Versorgungsspannungsberwachung
.equ	E_Taster	= 3		;Eingang Taster, low-aktiv
.equ	E_Geber_C	= 2		;Eingang Lichtschranke C vom Kassettenschacht
.equ	E_Geber_B 	= 1		;Eingang Lichtschranke B vom Segmentrad
.equ	E_Geber_A 	= 0		;Eingang Lichtschranke A vom Segmentrad

;*********************************************************************************************************
;***************  Verdrahtung Port C Ausgnge							**********
;*********************************************************************************************************
;PORT C
;BIT    7         6         5         4         3         2         1         0
;     nicht     Reset    Stelle 5  Stelle 4  Stelle 3  Stelle 2  Stelle 1  Nullstop
;   vorhanden              LSB                                     MSB     und  LED

.equ	A_Digit5	= 5		;Ausgang Treiber Digit 5 LSB	}
.equ	A_Digit4	= 4		;Ausgang Treiber Digit 4	}
.equ	A_Digit3	= 3		;Ausgang Treiber Digit 3	}high-aktiv
.equ	A_Digit2	= 2		;Ausgang Treiber Digit 2	}
.equ	A_Digit1	= 1		;Ausgang Treiber Digit 1 MSB	}
.equ	A_IR_LED	= 0		;Ausgang LEDs alle Lichtschranken	}low aktiv

.if (anzeigestellen==5)
.equ	alle_digits	= (1<<a_digit1)|(1<<a_digit2)|(1<<a_digit3)|(1<<a_digit4)|(1<<a_digit5)
.else
.equ	alle_digits	= 		(1<<a_digit2)|(1<<a_digit3)|(1<<a_digit4)|(1<<a_digit5)
.endif
;*********************************************************************************************************
;***************  Verdrahtung Port D								**********
;*********************************************************************************************************
.if (Geraet==1)			;Displayverdrahtung TC650M
.equ	Seg_g		= 7	;(auch mit 5-stelligem Testdisplay)
.equ	Seg_dp		= 6
.equ	Seg_c		= 5
.equ	Seg_d		= 4
.equ	Seg_e		= 3
.equ	Seg_a		= 2
.equ	Seg_f		= 1
.equ	Seg_b		= 0
.endif

.if (Geraet==2)			;Displayverdrahtung TC750
.equ	Seg_dp		= 7
.equ	Seg_e		= 6
.equ	Seg_c		= 5
.equ	Seg_b		= 4
.equ	Seg_f		= 3
.equ	Seg_d		= 2
.equ	Seg_g		= 1
.equ	Seg_a		= 0
.endif

.if (Geraet==3)			;Displayverdrahtung nach Schaltplan
.equ	Seg_dp		= 7
.equ	Seg_g		= 6
.equ	Seg_f		= 5
.equ	Seg_e		= 4
.equ	Seg_d		= 3
.equ	Seg_c		= 2
.equ	Seg_b		= 1
.equ	Seg_a		= 0
.endif

.equ	alle_segmente 	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)|(1<<Seg_dp)

;*********************************************************************************************************
;***************  Definition der Leuchtzeichen							**********
;*********************************************************************************************************
;***************  Ziffern									**********
;*********************************************************************************************************
.equ	Zeichen_0	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_1	=            (1<<Seg_b)|(1<<Seg_c)
.equ	Zeichen_2	= (1<<Seg_a)|(1<<Seg_b)|           (1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_3	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|                      (1<<Seg_g)
.equ	Zeichen_4	=            (1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_5	= (1<<Seg_a)|           (1<<Seg_c)|(1<<Seg_d)|           (1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_6	= (1<<Seg_a)|           (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
;.equ	Zeichen_6	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_7	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)
;.equ	Zeichen_7	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)
.equ	Zeichen_8	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_9	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|           (1<<Seg_f)|(1<<Seg_g)
;.equ	Zeichen_9	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)|(1<<Seg_g)

;*********************************************************************************************************
;***************  Buchstaben									**********
;*********************************************************************************************************
.equ	Zeichen_A	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_b_kl	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_C	= (1<<Seg_a)|                      (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_c_kl	=                                  (1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_d_kl	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_E	= (1<<Seg_a)|                      (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_F	= (1<<Seg_a)|                                 (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_G	= (1<<Seg_a)|           (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_H	=            (1<<Seg_b)|(1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_h_kl	=                       (1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_I	= Zeichen_1
.equ	Zeichen_i_kl	= (1<<Seg_c)
.equ	Zeichen_J	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)
.equ	Zeichen_j_kl	=                       (1<<Seg_c)|(1<<Seg_d)
.equ	Zeichen_L	=                                  (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_mikro	=            (1<<Seg_b)|                      (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_N	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_n_kl	=                       (1<<Seg_c)|           (1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_O	= Zeichen_0
.equ	Zeichen_o_kl	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_P	= (1<<Seg_a)|(1<<Seg_b)|                      (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_q_kl	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_r_kl	=                                             (1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_S	= Zeichen_5
.equ	Zeichen_t_kl	=                                  (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_U	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_u_kl	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)
.equ	Zeichen_Y	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|           (1<<Seg_f)|(1<<Seg_g)
;*********************************************************************************************************
;***************  Symbole, weitere Zeichen in der ASCII-Tabelle der Datei "daten.inc"		**********
;*********************************************************************************************************
.equ	Zeichen_punkt	= (1<<Seg_dp)
.equ	Zeichen_minus	=                                                                   (1<<Seg_g)

;*********************************************************************************************************
;***************  Zuweisungen der I/O-Adressen							**********
;*********************************************************************************************************
.equ	scaler1		= tccr1b	;Vorteiler Timer 1
.equ	scaler2		= tccr2		;Vorteiler Timer 2
.equ	timer2		= tcnt2		;Timer 2
.equ	match2		= ocr2		;Output-Compare-Register 2

;*********************************************************************************************************
;***************  Registerbelegung      							**********
;***************  Befehle, die mit den Low-Registern r0 bis r15 nicht mglich sind:		**********
;***************  SER, SBR, CBR, LDI, ANDI, ORI, CPI, SBCI, SUBI				**********
;*********************************************************************************************************
.undef xl			;       } 
.undef xh			;       } reduziert sptere
.undef yl			;       } Warnmeldungen des Assemblers
.undef yh			;       } 
;			= r0	;    } Ergebnis- und Hilfsregister
;			= r1	;    } Multiplikation
.def	mod0		= r2	; } LSB
.def	mod1		= r3	; }     40-Bit Register, fr Multiplikation und Radizierung
.def	mod2		= r4	; }	Registerbelegung "mod" nicht ndern, damit Routine
.def	mod3		= r5	; }	SQRT32 von Ruud v. Gessel ohne nderungen luft
.def	mod4		= r6	; } MSB
.def	s		= r7	;zum sichern des Statusregisters innerhalb Timer2 IRQ
.def	t		= r8	;zum sichern des Statusregisters innerhalb Timer1 IRQ
.def	null		= r9	;eine Null wird wirklich oft bentigt
.def	tBCD0		= r10	;BCD value digits 1 and 0
.def	tBCD1		= r11	;BCD value digits 3 and 2
.def	tBCD2		= r12	;BCD value digit  4
.def	ticker		= r13	;Zhler Ticks, aufwrtszhlend
.def	digit		= r14	;Zeiger Anzeigestelle, exklusiv in t2ovf-isr benutzt
.def	index_modus	= r15	;Zeiger Anzeigemodus
.def	var10		= r16	;Notizblock		} ergibt
.def	var11		= r17	;Hilfsregister 1	} ein bis zu
.def	var12		= r18	;Hilfsregister 2	} 32-Bit breites
.def	var13		= r19	;Hilfsregister 3	} Register
.def	var20		= r20	;	} ein
.def	var21		= r21 	;	} weiteres
.def	var22		= r22	;	} 32-Bit breites
.def	var23		= r23	;	} Register
.def	flags		= r24	;Status Laufwerk	} 
.def	flags1		= r25	;Status Berechnungen	} fnf
.def	flags2		= r26	;Benutzeroptionen	} FLAG
.def	flags3		= r27	;Menpunktezhlung	} Register
.def	flags4		= r28	;Fehlerspeicher		}
;.def			= r29	;frei
;	zl		= r30	;  } Zeiger Z
;	zh		= r31	;  } 
.def	lc		= r31	;Loopcounter fr Divisionsroutine "div32u"

;*********************************************************************************************************
;Index-Modus (Low-Register)
;BIT    7         6         5         4         3         2         1         0
;      frei      frei      frei  /	       Anzeigemodus als Index	       	  /
;				 /		   binr gezhlt		  /

;equ	frei		= 7	;
;equ	frei		= 6	;
;equ	frei		= 5	;
.equ	Modusbit_4	= 4	; } 
.equ	Modusbit_3	= 3	; } Anzeigemodi 1 bis "max_Karussell"
.equ	Modusbit_2	= 2	; } als Index binr gezhlt
.equ	Modusbit_1	= 1	; } 
.equ	Modusbit_0	= 0	; } 

.equ	alle_modusbits	= (1<<modusbit_4)|(1<<modusbit_3)|(1<<modusbit_2)|(1<<modusbit_1)|(1<<modusbit_0)

;*********************************************************************************************************
;Flags-Register
;BIT    7         6         5         4         3         2         1         0
;    synchron    Spul    Spulen    Wieder    letzter   letzter   Kennung     Kurz
;               beginn              gabe     Zustand   Zustand  verriegelt Kommando
;               erkannt  erkannt   erkannt   Licht B   Licht A            verriegelt

.equ	synchron	= 7	;Rechenwerk auf Kassette gelockt, alle Werte gltig
.equ	Spulbeginn	= 6	;Spulen hat jetzt gerade begonnen (Anforderung fr "Lagen setzen")
.equ	Spulen  	= 5	;Spulgeschwindigkeit erkannt
.equ	Wiedergabe	= 4	;Wiedergabegeschwindigkeit erkannt
.equ	Licht_B		= 3	;letzter Zustand der Lichtschranke B } mssen auf Bit 3 und
.equ	Licht_A		= 2	;letzter Zustand der Lichtschranke A } Bit 2 geroutet bleiben
.equ	Kennung_verr	= 1	;Anzeige Kennbuchstaben nach Schliessen des Kassettenfaches einmal gesetzt
.equ	Kurzkom_verr	= 0	;Tasterkommando kurz verriegelt

;*********************************************************************************************************
;Flags1-Register
;BIT    7         6         5         4         3         2         1         0
;     Zeit    Speicher    Dicke     Dicke     Dicke      Zeit      Zeit     Resync
;    Stempel    Init      valide    stabil    Delta   Differenz  schtzung  erfolgt
;     neu    angefordert                     negativ   negativ    erfolgt

.equ	Segment_neu	= 7	;neuer Zeitstempel Segmentvorbeilauf liegt vor
.equ	Init_Speicher	= 6	;Speicherinitialisierung angefordert
.equ	Dicke_valide	= 5	;einlaufende Dickenwerte sind innerhalb min- und max-Banddicke
.equ	Dicke_stabil	= 4	;Filterwert Banddicke hat geforderte max. nderungsrate unterschritten
.equ	Dicke_negativ	= 3	;Vorzeichen: neuer Filterwert U Banddicke < alter Filterwert U Bandicke
.equ	Zeit_negativ	= 2	;Vorzeichen: Sekundenwerte "T" < "S"
.equ	Zeitschaetzung	= 1	;Zeit whrend Messphase einmal gerechnet und gesetzt
.equ	Resync		= 0	;Nachsynchronisierung erfolgt

;*********************************************************************************************************
;Flags2-Register wird beim Abschalten ins EEPROM gerettet
;BIT    7         6         5         4         3         2         1         0
;     frei       Null      Kern      volle    zehntel    Start    Messen    Modus
;               unter    variabel   Stunden   Stunden    Modus   Anzeige   Bereich
;              drckung
;Bit 7 wre frei fr eine weitere zur Laufzeit schaltbare Bediener-Option
;oder einen Status, der ins EEPROM gerettet werden soll

;equ	frei		= 7	;frei				0 = 			1 = 
.equ	Nullunterdr	= 6	;Anzeige fhrender Nullen:	0 = anzeigen		1 = unterdrcken
.equ	Kern_variabel	= 5	;adaptive Kernradiusermittlung: 0 = aus			1 = aktiv
.equ	Stunden_volle	= 4	;Betriebsstundenanzeige H:	0 = Stunden / 10	1 = volle Stunden
.equ	Stunden_Zehntel	= 3	;Betriebsstundenanzeige h:	0 = volle Stunden	1 = Stunden x 10
.equ	Startmodus	= 2	;Anzeigemodus nach Einschalten:	0 = "T"			1 = zuletzt gewhlter
.equ	Calcanzeige	= 1	;Anzeige/Bedienung in Syncphase:0 = alles/normal	1 = "Calc"/gesperrt
.equ	volle_Modi	= 0	;Umfang Anzeigemodi:		0 = 1 bis min. Modus	1 = 1 bis max.Modus

.equ	alle_optionbits	= (1<<nullunterdr)|(1<<kern_variabel)|(1<<stunden_volle)|(1<<stunden_zehntel)|(1<<startmodus)|(1<<calcanzeige)|(1<<volle_modi)

;*********************************************************************************************************
;Flags3-Register
;BIT    7         6         5         4         3         2         1         0
;     frei      frei      frei      frei   /              Menpunkte               /
;                                          /             binr gezhlt             /
;                                          /            0 bis "max_Men"           /

;equ	frei		= 7	;
;equ	frei		= 6	;
;equ	frei		= 5	;
;equ	frei		= 4	;
.equ	menuebit_3	= 3	;    }
.equ	menuebit_2	= 2	;    } zur Nummerierung der
.equ	menuebit_1	= 1	;    } Menpunkte im Programmiermodus
.equ	menuebit_0	= 0	;    } 0 = kein Programmiermodus
				;die Menbits mssen rechtsbndig (ab Bit0) definiert bleiben

.equ	alle_menuebits	= (1<<menuebit_3)|(1<<menuebit_2)|(1<<menuebit_1)|(1<<menuebit_0)
;
;Mencodes	(S = Status: A = aus, E = ein)
;	P1-S volles Anzeigekarussell
;	P2-S Anzeige "calc" whrend initialer Messphase
;	P3-S Startmodus ist letzter Anzeigemodus
;	P4-S Betriebsstunden "h" mit einer Nachkommastelle
;	P5-S Betriebsstunden "H" / 10
;	P6-S Kernradius adaptiv, variabel
;	P7-S fhrende Nullen unterdrcken
;	P8-S ungenutzt
;	P-09 Anzeigehelligkeit ndern
;	P-10 Anzahl Modi im kleinen Karussell ndern
;	P-11 nicht rcksetzbare Betriebsstunden erhhen
;
;	auf der Anzeige sind ausschlielich einstellige Ziffern inkl. A - F darstellbar

;*********************************************************************************************************
;Flags4, gesamtes Byte ausschlielich fr Fehlermeldungen verwenden
;BIT    7         6         5         4         3         2         1         0
;    Watchdog    Ab-      Flash      ISR       Bank      Bank      Bank     EEPROM
;     Reset   schaltung   ende      Vektor      C         B         A       korrupt
;             angelaufen erreicht   falsch    defekt    defekt    defekt

.equ	fehler_8	= 7	;FE-8 Watchdog hat ausgelst
.equ	fehler_7	= 6	;FE-7 Abschaltroutine "sichere ins EEPROM" ausgefhrt
.equ	fehler_6	= 5	;FE-6 Programmcounter luft bis zum Flashende
.equ	fehler_5	= 4	;FE-5 falscher Sprung in die ISR Vektortabelle
.equ	fehler_4	= 3	;FE-4 ein oder mehrere fehlerhafte Bytes in EEPROM Bank C repariert
.equ	fehler_3	= 2	;FE-3 ein oder mehrere fehlerhafte Bytes in EEPROM Bank B repariert
.equ	fehler_2	= 1	;FE-2 ein oder mehrere fehlerhafte Bytes in EEPROM Bank A repariert
.equ	fehler_1	= 0	;FE-1 mehrere fehlerhafte Bytes im EEPROM, nicht sicher rekonstruiert

;*********************************************************************************************************
;**********	Belegung des SRAM Speicherbereiches						**********
;*********************************************************************************************************
.dseg				;347 + Stackpointer verwendet: 92+2*zebra*(mittelwerttiefe+delta_s)
anzeige:	.byte	5	;zur Vorhaltung der Ausgabezeichen fr die Anzeigestellen 1 bis 5
ausgabe:	.byte	5	;Inhalt von "Anzeige" wird nur zu Beginn des Multiplexrahmens bernommen und ausgegeben
fehleranzeige:	.byte	1	;anzuzeigende Fehlernummer
trimmanzeige:	.byte	1	;Leuchtsymbol zur Anzeige der Trimmrichtung des Framezhlers bei einem Zeitfehler
hell_index:	.byte	1	;Helligkeit der Anzeige in Prozent      	} Wichtung
hell_absolut:	.byte	1	;Helligkeit der Anzeige in Prozent * 2,55	} nach Tabelle
dimmer:		.byte	1	;Zhler auf- und abwrtslaufend fr wechselnde Helligkeit im "Dimmerbetrieb"
dimmer_min:	.byte	1	;unterer Helligkeitsgrenzwert
dimmer_max:	.byte	1	;oberer  Helligkeitsgrenzwert
kennung:	.byte	1	;Zhler aufwrtslaufend fr Dauer Kennbuchstabe und Lauftextgeschwindigkeit
blinker:	.byte	1	;Zhler aufwrtslaufend fr synchronisiertes Blinken und Faden im Display
blinkermaske:	.byte	1	;Bitmaske fr Position und Funktion Helligkeitssteuerung einzelner Displaystellen
karussell:	.byte	1	;Anzahl Anzeigemodi im kleinen Anzeigekarussell
zeiger_zeichen:	.byte	1	;Adresszeiger Zeichenketten
filterindex:	.byte	1	;Index der Filtertiefe Speicher "U", Banddicke
laufwerkpuls:	.byte	1	;Zhler aufwrtslaufend zur berwachung Laufwerksbewegung
stopimpuls:	.byte	1	;Zhler aufwrtslaufend zum Erzeugen eines langen Nullstopp-Impulses
taster_dauer:	.byte	1	;Zhler aufwrtslaufend zum Unterscheiden der Tasterkommandos "kurz" und "lang" und Prellen
syncfenster:	.byte	2	;Zhler aufwrtslaufend Sekunden * 2 * Zebra
frame:		.byte	1	;Zhler aufwrtslaufend Frames (1 Frame = 40 ms)
sekunde:	.byte	1	;Zhler aufwrtslaufend Sekunden
scratch:	.byte	4	;32-Bit Zwischenablage
scratch1:	.byte	3	;24-Bit Zwischenablage
kernradius:	.byte	3	;Wickelkernradius in mm / 100.000
wickelradius:	.byte	2	;Wickelradius in mm / 100
umdrehungszahl:	.byte	2	;Wickeltellerumdrehungen * 2 * Zebra, rcksetzbar
lagenzahl:	.byte	2	;Bandlagen * 2 * Zebra
t_segm_init:	.byte	2	;mittlere Dauer Segmentlauf letzte Umdrehung, exclusiv zur Speicherinitialisierung genutzt
t_segm_akt:	.byte	2	;aktuelle Dauer Segmentlauf
t_segm:		.byte	2	;mittlere Dauer Segmentlauf aktuelle bis Mittelwerttiefe alte Umdrehung
t_segm_delta:	.byte	2	;mittlere Dauer Segmentlauf delta_s bis delta_s+mittelwerttiefe alte Umdrehung
;dicke_init:	.byte	2	;separat vorgehaltene Banddicke fr eine Speicherinitialisierung
dicke_roh:	.byte	2	;Banddickenwert, wie einlaufend, ungefiltert
dicke:		.byte	4	;Speicher "U", Dickenwert in 0,01 m; 2 Byte Ganzzahlanteil, 2 Byte Nachkommastellen
dicke_delta:	.byte	4	;Banddicke_neu minus Banddicke_alt, Vorzeichen als Flag "Dicke_negativ" in "Flags1"
dicke_delta_anz:.byte	2	;"Dicke_delta" exklusiv fr Anzeigezwecke in nm umgerechnet
zaehler_dicke:	.byte	1	;Umdrehungszhler, aufwrtslaufend: Ableitung Banddicke ist ruhig? zur Entscheidung "synchron"
zeit_calc:	.byte	2	;abgespielte Zeit in Sekunden, errechnet
zeit_quartz:	.byte	2	;angezeigte Zeit in Sekunden, quarzgefhrt
zeit_delta:	.byte	2	;Betrag der Abweichung gerechnete zu angezeigter Zeit, Vorzeichen ist "Zeit_negativ" in "Flags1"
stunden:	.byte	3	;Gesamtbandlaufzeit in Stunden * 3600 / Kopfintervall (Kopfintervall = 8)
stunden_anz:	.byte	2	;Anzeige der Gesamtbandlaufzeit in Stunden oder Stunden / 10
stunden_r:	.byte	3	;rcksetzbare Bandlaufzeit in Stunden * 3600 / Kopfintervall (Kopfintervall = 8)
stunden_r_anz:	.byte	2	;Anzeige der rcksetzbaren Bandlaufzeit in Stunden oder Stunden * 10
icr:		.byte	2	;Input-Capture-Wert
icr_alt:	.byte	2	;Input-Capture-Wert n-1
debug:		.byte	2	;Debug-Wert, zur bergabe eines Anzeigewertes an beliebiger Programmstelle
zeiger_s:	.byte	2	;Zeiger zum Befllen und Auslesen des Ringspeichers "S"
speicher_s:	.byte	2*zebra*(mittelwerttiefe+delta_s)	;Ringspeicher "S", Segmentvorbeilufe

;*********************************************************************************************************
;SRAM-Zelle "Blinkermaske"
;BIT    7         6         5         4         3         2         1         0
;     Dimmer    Dimmer   Digit 5   Digit 4   Digit 3   Digit 2   Digit 1     frei
;      ein    verriegelt   fr die Helligkeitsmanipulation selektiert
;

.equ	Dimmer_ein	= 7	;1 = dimmen, 0 = blinken (gilt fr alle unten selektierten Digits)
.equ	Dimmer_sync	= 6	;Blinkerzhler wurde synchronisiert (fr "calc"- und Menanzeige genutzt)
;			= 5	;Digit 5  } 
;			= 4	;Digit 4  } fr die Helligkeitssteuerung
;			= 3	;Digit 3  } selektierte Digits
;			= 2	;Digit 2  } 
;			= 1	;Digit 1  } 
;	frei		= 0	;

;*********************************************************************************************************
;**********	Sprungadressen fr die Interrupts						**********
;*********************************************************************************************************
.cseg
.org	0x0000
	rjmp reset		;Reset
	rjmp fehler_isr_vektor	;External IRQ 0
	rjmp fehler_isr_vektor	;External IRQ 1
	rjmp t2ocm		;Timer 2 Compare 
	rjmp t2ovf		;Timer 2 Overflow
	rjmp t1cev		;Timer 1 Capture Event
	rjmp fehler_isr_vektor	;Timer 1 Compare A
	rjmp fehler_isr_vektor	;Timer 1 Compare B
	rjmp fehler_isr_vektor	;Timer 1 Overflow
	rjmp fehler_isr_vektor	;Timer 0 Overflow
	rjmp fehler_isr_vektor	;SPI Transfer Complete
	rjmp fehler_isr_vektor	;UART Rx Complete
	rjmp fehler_isr_vektor	;UART Data Register Empty
	rjmp fehler_isr_vektor	;UART Tx Compplete
	rjmp fehler_isr_vektor	;ADC Conversion Complete
	rjmp fehler_isr_vektor	;EEPROM Ready
	rjmp fehler_isr_vektor	;Analog Comparator
	rjmp fehler_isr_vektor	;TWI
	rjmp fehler_isr_vektor	;SPM ready	

;*********************************************************************************************************
;**********	Initialisierung 		 						**********
;*********************************************************************************************************
reset:
	ldi	flags4,(1<<fehler_8)	;
	in	var10,mcucsr		;
	sbrs	var10,wdrf		;auer bei einem Watchdog-Reset
	clr	flags4			;alle Fehlercodes rcksetzen
reset_nach_fehler:
	ldi	var10,high(RAMend)	;Position Stackpointer festlegen
	out	sph,var10		;
	ldi	var10,low(RAMend)	;ist Pflicht bei Anwendung von ISRs
	out	spl,var10		;
	clr	null			;eine "Null" bereitstellen
;**********	EEPROM initialisieren  *******************************************************************
	rcall	teste_eeprom		;EEPROM auf Korruption durchsuchen und reparieren
	ldi	var11,adr_eeprom	;wenn in der erste Zelle
	rcall	eeread			;
	ldi	var11,0xaa		;"AA" steht
	cpse	var10,var11		;ist das EEPROM schon initialisiert
	rcall	init_eeprom		;sonst EEPROM initialisieren
;**********	nicht bentigte Resourcen abschalten  ****************************************************
	out	ADCSRA,null		;AD-Wandler abschalten
	ldi	var10,(1<<ACD)		;
	out	ACSR,var10		;analog Komparator abschalten
	ldi	var10,(1<<WDCE)|(1<<WDE);
	out	WDTCR,var10		;Watchdog-Timer zum Schalten vorbereiten
	ldi	var10,(1<<WDE)|(1<<WDP2)|(1<<WDP1)
	out	WDTCR,var10		;Watchdog-Timeout 1 Sekunde
;**********	Timer, Vorteiler und Timer-IRQs konfigurieren  *******************************************
	ldi	var10,(1<<CS21)		;Prescaler Timer2 auf CK/8 einstellen
	out	scaler2,var10		;
	ldi	var10,(1<<TICIE1)|(1<<TOIE2)|(1<<OCIE2)
	out	timsk,var10		;Input1Capture-, Timer2-Overflow- und Compare2-Match-IRQ freigeben
	ldi	var10,(1<<ICNC1)|(1<<ICES1)|(1<<CS11)|(1<<CS10)	;Prescaler Timer1 auf CK/64
	out	scaler1,var10		;Capture-Event auf steigende Flanke mit NoiseCanceler
;**********	Portpins definieren  *********************************************************************
	ldi	var10,(1<<a_stop)	;
	out	ddrb,var10		;alle Pins PortB Eingang auer Nullstopausgang
	sbi	portb,e_taster		;PullUp Tasteneingang setzen
	ldi	var10,alle_digits|(1<<a_ir_led)
	out	ddrc,var10		;setzen der Portpins C als Ausgang
	ldi	var10,alle_segmente	;
	out	ddrd,var10		;setzen der Portpins D als Ausgang zur Steuerung der Kathoden
;**********	Ausgnge bedienen  ***********************************************************************
.if (gem_anode==1)			;fr Anzeigen mit gemeinsamer Anode
	ldi	var10,~a_digit5		;initialisiere ein Digit
	ldi	var11,alle_segmente	;
.else
	ldi	var10,a_digit5		;initialisiere ein Digit
	ldi	var11,~alle_segmente	;
.endif
	out	portc,var10		;
	cbi	portc,a_ir_led		;IR-LEDs einschalten
	out	portd,var11		;alle Segmente abschalten
;**********	gerettete Werte holen  *******************************************************************
	rcall	loesche_sram		;mal nass durchwischen
	rcall	lade_eeprom		;lade Daten aus dem EEPROM
;**********	Das Drehgeberpolling muss nach dem Einschalten eine "0" ausgeben  ************************
	in	flags,pinb		;sichere Eingangsstatus
	andi	flags,(1<<e_geber_b)|(1<<e_geber_a)	;maskiere Nutzdaten der Lichtschranken aus
	lsl	flags			;
	lsl	flags			;neuer Zustand wird hier auch alter Zustand
;**********	Register vorbereiten  ********************************************************************
	clr	flags1			;
	sbrs	flags2,startmodus	;
	clr	index_modus		;
	clr	flags3			;kein aktives Men
	clr	digit			;
	sts	blinkermaske,null	;
	ldi	var10,filterindex_1	;
	sts	filterindex,var10	;
	sts	syncfenster,null	;
	sts	syncfenster+1,null	;
	rcall	init_helligkeit		;Variablen fr Helligkeitssteuerung Anzeige initialisieren
;**********	Behandlung Meneinstieg  *****************************************************************
	sbic	pinb,e_taster		;wenn der Taster beim Einschalten gedrckt ist
	rjmp	ende_menueauswahl	;
	tst	flags4			;UND keine Fehlermeldung anliegt
	brne	ende_menueauswahl	;
	ldi	flags3,(1<<menuebit_0)	;in Menpunkt 1 schalten
	sbr	flags,(1<<kurzkom_verr)	;Kurzkommando verriegeln
	ldi	var10,255-40		;Tasterkommandos fr 40 Frames sperren
	sts	taster_dauer,var10	;
ende_menueauswahl:
;*********************************************************************************************************
	rcall	init_kernradius		;Wickelkernradius auf Defaultwert setzen
	rcall	rechne_stunden		;einmal Betriebsstunden fr die Anzeige rechnen
;	rcall	zaehle_radius		;Radius auf Minimum setzen
;**********	Ende  Initialisierung  *******************************************************************
	sei				;Interrupts freigeben

;*********************************************************************************************************
;***************  Programmstart									**********
;*********************************************************************************************************
start:
	sbis	pinb,e_spannung		;wenn die Versorgungsspannung abnimmt
	rcall	rette_werte		;alle LEDs abschalten und Werte ins EEPROM retten
;*********************************************************************************************************
;***************  Den Zustand der beiden Lichtschranken am Aufwickelteller mit dem vorherigen	**********
;***************  Zustand vergleichen. Bei einer nderung die Drehrichtung feststellen und	**********
;***************  den Umdrehungszhler, den Lagenzhler und den Laufwerksherzschlag bedienen	**********
;*********************************************************************************************************
	rcall	hole_drehgeber		;Drehgeber pollen
;*********************************************************************************************************
;***************  Jeden neuen Wert aus der Timer-1 Input-Capture-ISR mit dem vorherigen		**********
;***************  verrechnen. Je nach ermittelter Tellerdrehzahl die entsprechenden Flags fr	**********
;***************  den Laufwerkstatus bedienen.							**********
;*********************************************************************************************************  
	sbrs	flags1,segment_neu	;jeden neuem Input-Capture-Wert
	rjmp	ende_capture		;
	rcall	capture_event		;bewerten und Laufwerkstatus signalisieren
	cbr	flags1,(1<<segment_neu)	;Auftrag erledigt
	sbrs	flags,wiedergabe	;im Play-Modus
	rjmp	ende_capture		;
	rcall	in_speicher_s		;alle Messwerte im Speicher "S" ablegen
;*********************************************************************************************************
;***************  Ableitung der Banddicke ermitteln, bei erreichen des Grenzwertes unter	**********
;***************  Beachtung des Synchronisierfensters Flag "dicke_stabil" setzen und damit	**********
;***************  Zeitberechnung und Zeitbergabe anfordern					**********
;********************************************************************************************************* 
	sbrc	flags1,dicke_valide	;wenn einlaufende Dickenwerte gltig sind
	rcall	validiere_speicher_u	;Dickenwerte validieren
ende_capture:
;*********************************************************************************************************
;**********	Fallbehandlung: Spulbeginn direkt nach Wiedergabe				**********
;*********************************************************************************************************
	sbrs	flags,spulbeginn	;zum Beginn des Spulens
	rjmp	kein_spulbeginn		;
	rcall	setze_lagenzahl		;Lagenzahl so setzen, dass kein Sprung in der Zeitanzeige entsteht
	cbr	flags,(1<<spulbeginn)	;
kein_spulbeginn:
;*********************************************************************************************************
;**********	Ticker wird vom Timerinterrupt mit 2000 Hz hochgezhlt				**********
;*********************************************************************************************************
	mov	var10,ticker		;sichere Ticker
	cpi	var10,80		;prfe ob 1 Frame um
	brsh	ein_frame_mehr		;
	rjmp	start			;wenn nicht, Zeitzhlung verlassen
ein_frame_mehr:
	clr	ticker			;Ticker rcksetzen
;*********************************************************************************************************
;**********	Durchlauf hier jedes Frame							**********
;*********************************************************************************************************
;**********	Behandlung "Blinker"								**********
;**********	steuert alle 32 Frames die Umkehr der Laufrichtung "Dimmer" und alle		**********
;**********	16 Frames das Absenken auf 25% Helligkeit im Blinkmodus				**********
;*********************************************************************************************************
	lds	var10,blinker		;
	inc	var10			;Blinker bedienen
	sts	blinker,var10		;jeder Prozess kann ihn durch Nullen fr sich synchronisieren
;*********************************************************************************************************
;**********	Behandlung "Kennung"								**********
;**********	Von 0 bis "Scrollintervall" wird nach Moduswechsel der Kennbuchstabe gezeigt	**********
;**********	Im Textmodus wird die Laufgeschwindigkeit hiermit gesteuert			**********
;*********************************************************************************************************
	lds	var10,kennung		;
	cpi	var10,255		;den Zeichenfolgezhler
	breq	ende_kennung		;
	inc	var10			;bedienen
	sts	kennung,var10		;
ende_kennung:
;*********************************************************************************************************
;**********	Dauer des Nullstopp-Impulses							**********
;*********************************************************************************************************
	lds	var10,stopimpuls	;
	cpi	var10,255		;Zhler Nullstoppdauer
	breq	kein_null_impuls	;
	inc	var10			;bedienen
	sts	stopimpuls,var10	;
	rjmp	ende_nullstop		;
kein_null_impuls:
	cbi	portb,a_stop		;sonst Nullstopp-Ausgang deaktivieren
ende_nullstop:
;*********************************************************************************************************
;**********	Auf- und abwrtslaufenden Zhler "dimmer" fr Helligkeitssteuerung		**********
;**********	bereitstellen. Der Wert luft von 0 bis "max. Helligkeit" und zurck.		**********
;**********	benutzte Register: var1[3:0], dimmer						**********
;*********************************************************************************************************
	lds	var10,dimmer		;hole aktuelle Helligkeit
	lds	var11,blinker		;
	lds	var12,dimmer_min	;unteren und
	lds	var13,dimmer_max	;oberen Grenzwert laden
	sbrs	var11,5			;alle 2^5 Frames Richtungswechsel
	rjmp	dimmer_heller		;heller gehts da lang
dimmer_dunkler:
	cp	var10,var12		;Minimum erreicht?
	brlo	dimmer_heller		;
	sub	var10,var12		;sonst etwas dunkler bitte
	rjmp	ende_dimmer		;
dimmer_heller:
	cp	var10,var13		;Maximum erreicht?
	brsh	dimmer_dunkler		;ja, mach dunkler
	add	var10,var12		;etwas heller bitte
ende_dimmer:
	sts	dimmer,var10		;
;*********************************************************************************************************
;**********	Tasterbehandlung								**********
;**********	Bei gedrcktem Taster wird der Zhler "taster_dauer" bedient.			**********
;**********	Nach 1 Sekunde wird ein "Langkommando ausgefhrt, der Zhler genullt und	**********
;**********	ein "Kurzkommando" verriegelt.							**********
;**********	Wird der Taster vor Ablauf einer Sekunde losgelassen, dann werden bei		**********
;**********	Prellen ("taster_dauer" < 2 Frames) Zhler und Flag genullt. Sonst wird wenn	**********
;**********	entriegelt ist ein "Kurzkommando" ausgefhrt, verriegelt und der Zhler	genullt.**********
;**********	Entriegelt wird 1 Frame nach einem Kurzkommando.				********** 
;**********	Damit ist der Taster wieder "scharf".						**********
;*********************************************************************************************************
	lds	var10,taster_dauer	;
	sbic	pinb,e_taster		;
	rjmp	teste_auf_kurz		;
;*********************************************************************************************************
;**********	Taster ist gedrckt								**********
;*********************************************************************************************************
	inc	var10			;Zhler zum Messen der Dauer des Tastendrucks bedienen
	sts	taster_dauer,var10	;
	cpi	var10,25		;Taster schon eine Sekunde lang gedrckt?
	brne	ende_taster		;
	sts	taster_dauer,null	;dann Zhler rcksetzen
	rcall	kommando_lang		;und Tasterkommando "lang" ausfhren
	sbr	flags,(1<<kurzkom_verr)	;Kurzkommando verriegeln
	sts	taster_dauer,null	;
	rjmp	ende_taster		;
;*********************************************************************************************************
;**********	Taster ist nicht oder nicht mehr gedrckt					**********
;*********************************************************************************************************
teste_auf_kurz:
	cpi	var10,prellen		;mgliche Prellzeit um?
	brlo	entriegeln		;nein, raus
	sbrc	flags,kurzkom_verr	;wenn entriegelt ist
	rjmp	teste_entriegeln	;
	rcall	kommando_kurz		;Tasterkommando "kurz" ausfhren
	sbr	flags,(1<<kurzkom_verr)	;und verriegeln
	ldi	var10,prellen+8		;Zhler manipulieren (ich wei nicht mehr warum genau)
	sts	taster_dauer,var10	;
	rjmp	ende_taster		;
teste_entriegeln:
	inc	var10			;
	sts	taster_dauer,var10	;
	cpi	var10,prellen+8		;
	brlo	ende_taster		;
entriegeln:
	cbr	flags,(1<<kurzkom_verr)	;Kurzkommando freigeben
	sts	taster_dauer,null	;und alles auf Anfang
ende_taster:
;*********************************************************************************************************
;**********	Kassettenfachberwachung							**********
;*********************************************************************************************************
	sbic	pinb,e_geber_c		;wenn das Kassettenfach offen ist
	rjmp	kassettenfach_zu	;
;kassettenfach_auf:			;Laufwerkstatus lschen
	cbr	flags,(1<<synchron)|(1<<kennung_verr)
	sts	lagenzahl,null		;
	sts	lagenzahl+1,null	;einige weitere Werte nullen
	sts	zeit_calc,null		;
	sts	zeit_calc+1,null	;
	sts	zeit_quartz,null	;
	sts	zeit_quartz+1,null	;
	sts	zeit_delta,null		;
	sts	zeit_delta+1,null	;
	cbr	flags1,(1<<zeitschaetzung)|(1<<resync)|(1<<zeit_negativ)
	ldi	var10,filterindex_1	;
	sts	filterindex,var10	;
	rcall	init_kernradius		;Wickelkernradius auf Defaultwert setzen
;	rcall	zaehle_radius		;Radius auf Minimum setzen
	rjmp	kassettenfach_offen	;
kassettenfach_zu:
;*********************************************************************************************************
;**********	nach dem Schliessen des Kassettenfaches kurz den Kennbuchstaben anzeigen	**********
;**********	Ist bei 5-stelliger Anzeige nur im Modus "T", Zeitanzeige, wirksam		**********
;*********************************************************************************************************
	sbrc	flags,kennung_verr	;nur einmal
	rjmp	kassettenfach_offen	;
	sbr	flags,(1<<kennung_verr);
	sts	kennung,null		;Anzeige Kennbuchstaben anfordern
	sts	zeiger_zeichen,null	;Lauftext fngt vorne an
kassettenfach_offen:
;*********************************************************************************************************
;**********	stehendes Bandlaufwerk erkennen							**********
;*********************************************************************************************************
	lds	var10,laufwerkpuls	;
	cpi	var10,laufwerk_dreht	;ohne "Herzschlag" steht das Laufwerk
	brsh	laufwerk_steht		;dann hier lang
	inc	var10			;
	sts	laufwerkpuls,var10	;
	sbrc	flags,wiedergabe	;
	rjmp	ende_laufwerk		;bei Wiedergabe hier lang
;*********************************************************************************************************
;**********	im Spulmodus wird der Lagenzhler vom Wickelteller mitgefhrt (nicht gerechnet)	**********
;**********	und daraus die abgelaufene Zeit gerechnet und an die Anzeige bergeben.		**********
;*********************************************************************************************************
;laufwerk_spult:
	rcall	sekunden_absolut	;errechne die abgespielten Sekunden aus Lagen und Dicke
	rcall	uebergebe_zeit		;errechnete Zeit sofort komplett an die Anzeige bergeben
	rcall	zaehle_radius		;Radius aus Banddicke und Bandlagen ermitteln
	rjmp	laufwerk_spult_oder_steht;und raus
laufwerk_steht:
	cbr	flags,(1<<spulbeginn)|(1<<spulen)|(1<<wiedergabe)
	rcall	setze_lagenzahl		;aus der Zeitanzeige die Soll-Lagenzahl ermitteln
laufwerk_spult_oder_steht:
	sbr	flags1,(1<<init_speicher);Speicher S Initialisierung anfordern
	lds	var10,blinkermaske	;
	cbr	var10,(1<<dimmer_sync)	;Synchronisierung Dimmer anfordern
	sts	blinkermaske,var10	;
	sts	zeiger_s,null		;
	sts	zeiger_s+1,null		;
	sts	syncfenster,null	;ein angefangener Messzyklus beginnt neu
	sts	syncfenster+1,null	;
	lds	var10,trimmanzeige	;sichere Zeichenausgabe "Trimmanzeige"
	andi	var10,zeichen_minus	;Minuszeichen nicht anfassen, Rest lschen
	sts	trimmanzeige,var10	;und wieder weglegen
ende_laufwerk:
;*********************************************************************************************************
;**********	Behandlung neues Displaybild							**********
;*********************************************************************************************************
;**********	Fehleranzeige bedienen								**********
;*********************************************************************************************************
	tst	flags4			;bei anliegendem Fehlercode
	breq	ende_fehleranzeige	;
	lds	var10,fehleranzeige	;
	tst	var10			;und schon gesetzter Fehlernummer
	breq	setze_fehlernummer	;
	rjmp	zeige_fehlercode	;Anzeige entsprechend bedienen, sonst
;*********************************************************************************************************
;**********	Kleinstes gesetztes Fehlerflag ermitteln (= hchstwertiger Fehler)		**********
;**********	und zugehrige Fehlernummer in "Fehleranzeige" sichern.				**********
;**********	benutzte Register: var1[1:0], fehleranzeige					**********
;*********************************************************************************************************
setze_fehlernummer:
	mov	var10,flags4		;Fehlerspeicher sichern
	ldi	var11,1			;Schleifenzhler initialisieren
schleife_fehlernummer:
	sbrc	var10,0			;niederwertigstes Bit testen
	rjmp	fehlernummer_gefunden	;hchstwertigen Fehlercode gefunden
	lsr	var10			;
	inc	var11			;
	cpi	var11,9			;
	brlo	schleife_fehlernummer	;
	rjmp	ende_fehleranzeige	;
fehlernummer_gefunden:
	sts	fehleranzeige,var11	;
ende_fehleranzeige:
;*********************************************************************************************************
;**********	Menanzeige bedienen								**********
;*********************************************************************************************************
	mov	var10,flags3		;sichere Men
	andi	var10,alle_menuebits	;maskiere Men
	breq	kein_menue		;
	rjmp	zeige_menue		;Anzeige entsprechend bedienen
kein_menue:
;*********************************************************************************************************
;**********	Kassettenfach abfragen								**********
;*********************************************************************************************************
	sbis	pinb,e_geber_c		;wenn das Kassettenfach offen ist
	rjmp	zeige_cass		;"CASS" zeigen
;*********************************************************************************************************
;**********	"Calc"-Anzeige bedienen								**********
;*********************************************************************************************************
	sbrs	flags,wiedergabe	;nur bei Aufnahme oder Wiedergabe
	rjmp	zeige_kein_calc		;
	sbrs	flags2,calcanzeige	;UND bei Option P2-E
	rjmp	zeige_kein_calc		;
	sbrc	flags,synchron		;UND solange nicht synchron
	rjmp	zeige_kein_calc		;
	sbrc	flags1,init_speicher	;UND nach einer Speicherinitialisierung
	rjmp	zeige_kein_calc		;
;*********************************************************************************************************
;**********	Dimmen der "CALC"-Anzeige synchronisieren					**********
;*********************************************************************************************************
	lds	var10,blinkermaske	;
	sbrc	var10,dimmer_sync	;wenn Blinkersynchronisierung freigegeben ist
	rjmp	ende_blinker_sync	;
	sbr	var10,(1<<dimmer_sync)	;weitere Versuche sperren und unten den Sync ausfhren
	sts	blinkermaske,var10	;
	lds	var10,hell_absolut	;Maximalhelligkeit laden und
	sts	dimmer,var10		;
	ldi	var10,32		;Blinkerphase auf maximale Helligkeit schieben
	sts	blinker,var10		;
ende_blinker_sync:
	rjmp	zeige_calc		;"CALC" auf die Anzeige
zeige_kein_calc:
;*********************************************************************************************************
;**********	ab hier in die Modi verzweigen							**********
;**********	Fehler-, Men- und Calc-Anzeige wurden oben schon abgehandelt			**********
;*********************************************************************************************************
	sts	blinkermaske,null	;jedes Blinken beenden
	mov	var10,index_modus	;sichere Register
	andi	var10,alle_modusbits	;Nutzdaten Anzeigemodus maskieren
	cpi	var10,max_karussell	;
	brlo	modus_gueltig		;
	ldi	var10,~alle_modusbits	;bei ungltigen Modi in Anzeigemodus "T" verzweigen
	and	index_modus,var10	;
	clr	var10			;
modus_gueltig:
	ldi	zl,byte1(modus)		;Sprungadresse laden
	ldi	zh,byte2(modus)		;
	add	zl,var10		;Index dazu
	adc	zh,null			;
	ijmp				;springe zum angewhlten Anzeigemodus
;*********************************************************************************************************
;**********	Alle Routinen "zeige_x" liegen in der Include-Datei "Anzeigeroutinen.inc"	**********
;**********	und springen nach Ablauf zum Label "ende_displaybild" zurck.			**********
;**********	Um die Position von "zeige_umdrehungen", "zeige_kopfstunden_r", "zeige_text"	**********
;**********	im Karussel zu ndern mssen die entsprechenden Konstanten "Pos_N" angepasst	**********
;**********	werden, da genau fr diese Positionen Tasterkomandos oder die Unterdrckung des	**********
;**********	 Kennbuchstabens hinterlegt sind.						**********
;*********************************************************************************************************
modus:					;Position
	rjmp	zeige_zeit		; 1	"t STUNDEN:MINUTEN:SEKUNDEN" quarzgesteuert, nachgefhrt
	rjmp	zeige_umdrehungen	; 2	"U UMDREHUNGEN",               fr Positionswechsel Pos_U anpassen
	rjmp	zeige_kopfstunden_r	; 3	"h KOPFSTUNDEN", rcksetzbar,  fr Positionswechsel Pos_h anpassen
	rjmp	zeige_kopfstunden	; 4	"H GESAMTKOPFSTUNDEN"
	rjmp	zeige_text		; 5	"  LAUFTEXT" max. 254 Zeichen, fr Positionswechsel Pos_L anpassen
	rjmp	zeige_dicke_3		; 6	" BANDDICKE" mit 1 Nachkommastelle und ""-Zeichen
	rjmp	zeige_dicke_0		; 7	"b BANDDICKE" einlaufender ungefilterter Wert
	rjmp	zeige_dicke_4		; 8	" BANDDICKE" mit 2 Nachkommastellen
	rjmp	zeige_dicke_delta	; 9	"A ABLEITUNG BANDDICKE" ber 1 Feld in nm mit 2 Nachkommastellen
	rjmp	zeige_filterindex	;10	"I FILTERINDEX"
	rjmp	zeige_syncfenster	;11	"F SYNCFENSTER"
	rjmp	zeige_bandlagen		;12	"L BANDLAGEN"
	rjmp	zeige_sekunden		;13	"S.SEKUNDEN" gerechnet
	rjmp	zeige_sekundenfehler	;14	"E FEHLER" gerechnete Sekunden zu Zeitanzeige
	rjmp	zeige_segmentlaufzeit	;15	"J LETZTE SEGMENTLAUFZEIT"
	rjmp	zeige_drehzahl		;16	"n SEGMENTLAUFZEIT" Mittelwert der letzten Mittelwerttiefe*Umdrehungen
	rjmp	zeige_drehzahl_delta	;17	"N SEGMENTLAUFZEIT" Mittelwert von delta_s alten Mittelwerttiefe*Umdrehungen
	rjmp	zeige_kernradius	;18	"o KERNRADIUS" angewendeter Kernradius
	rjmp	zeige_wickelradius	;19	"r WICKELRADIUS"
	rjmp	zeige_debug		;20	"d DEBUGWERT"
ende_displaybild:
;*********************************************************************************************************
;**********	Bedienung Framezhler								**********
;*********************************************************************************************************
	wdr				;ich lebe noch
	lds	var10,frame		;hole Frames
	inc	var10			;ein Frame mehr
	sts	frame,var10		;Frames sichern
	cpi	var10,25		;prfe ob 25 Frames vorbei
	brne	ende_zeit		;
	sts	frame,null		;Frames zurcksetzen
;*********************************************************************************************************
;**********	Durchlauf hier jede Sekunde							**********
;*********************************************************************************************************
;**********	Die Initialisierung des Speicher "S" wird angefordert von Stopp und Spulen.	**********
;**********	Ausgefhrt wird sie nur bei Wiedergabe und wenn nach Ablauf einer Sperre	**********
;**********	nach einer vollen Umdrehung ein valider Startwert vorliegt.			**********
;*********************************************************************************************************
	sbrs	flags,wiedergabe	;nur im Play-Modus
	rjmp	ende_ablaufsteuerung	;

	sbrs	flags1,init_speicher	;UND nur wenn eine Initialisierung Speicher S angefordert ist
	rjmp	keine_initialisierung	;

	lds	var10,zeiger_s		;
	lds	var11,zeiger_s+1	;
	ldi	var12,byte2(feldersperre+2*zebra)	;UND nur nach der ersten gltigen Umdrehung
	cpi	var10,byte1(feldersperre+2*zebra)	;zur Gewinnung des Startwertes zur Initialisierung
	cpc	var11,var12		;
	brlo	keine_initialisierung	;

	rcall	init_speicher_s		;Speicher S mit errechneten, theoretischen Werten befllen
	cbr	flags1,(1<<init_speicher);erledigt
	sbrc	flags1,zeitschaetzung	;nach der Speicherinitialisierung einmal eine
	rjmp	keine_zeitschaetzung	;
;*********************************************************************************************************
;**********	Eine vorlufige Zeit kann anhand der jetzigen Banddicke und Segmentlaufzeit	**********
;**********	errechnet werden. Das geschieht nur einmal nach einem Bandwechsel.		**********
;*********************************************************************************************************
;zeitschaetzung:
	rcall	bilde_mittelwert_t	;vorher Mittelwerte bilden, sonst geht das in die Hose!
	rcall	lagen_absolut		;
	rcall	sekunden_absolut	;eine Spielzeit ermitteln und
	rcall	uebergebe_zeit		;bergeben (im Modus "CALC" anzeigen sieht man davon nichts)
	sbr	flags1,(1<<zeitschaetzung);erledigt
	rjmp	quarzzeit_bitte		;
keine_zeitschaetzung:
keine_initialisierung:
	lds	var10,syncfenster	;erst nach der kurzen Syncsperre
	lds	var11,syncfenster+1	;die Berechnungen beginnen
	cpi	var10,byte1(syncsperre_kurz*k_0);k_0=zebra*2
	ldi	var12,byte2(syncsperre_kurz*k_0);
	cpc	var11,var12		;das vermeidet fehlerhafte Lageneintrge und
	brlo	quarzzeit_bitte		;damit auch eine fehlerhafte Zeitschtzung
;*********************************************************************************************************
;**********	Es wird ein Banddickenwert errechnet in den Filter U geschrieben und beruhigt.	**********
;**********	Sobald sich ein stabiler Wert Dicke eingestellt hat (Flag: "dicke_stabil")	**********
;**********	werden daraus die absoluten Lagen und absoluten Sekunden errechnet und einmal	**********
;**********	an das Uhrwerk bergeben.(Flag: "synchron") Die Filtertiefe U wird jetzt	**********
;**********	deutlich angehoben. Ab jetzt kann die Quarzzeit nur noch sukzessive an die im	**********
;**********	Hintergrund weiterhin errechnete Spielzeit nachgefhrt werden.			**********
;*********************************************************************************************************
	rcall	lagen_absolut		;absoluten Bandwickellagen aus Tellerdrehzahl und Dickenwert errechnen
	rcall	sekunden_absolut	;und die abgespielten Sekunden aus Lagenzahl und Dickenwert errechnen
	sbrs	flags1,dicke_stabil	;bei eingependeltem Dickenwert
	rjmp	quarzzeit_bitte		;
	sbrc	flags,synchron		;
	rjmp	quarzzeit_bitte		;
	rcall	uebergebe_zeit		;die errechnete Zeit einmal bergeben
	sbr	flags,(1<<synchron)	;erledigt
	sts	zeiger_zeichen,null	;nach "calc"-Anzeige wrde Lauftext vorne anfangen
	sbrc	flags2,calcanzeige	;bei gewhlter Option
	sts	kennung,null		;kurz den Kennbuchstaben zeigen
quarzzeit_bitte:
	rcall	synchronisiere_zeit	;generiere Quarzzeit; nach Sync fhre sie der errechneten Zeit nach
ende_ablaufsteuerung:
;*********************************************************************************************************
	lds	var10,sekunde		;
	inc	var10			;Sekunde erhhen
	sts	sekunde,var10		;
	cpi	var10,kopfintervall	;Betriebsstundenintervall erreicht
	brlo	ende_zeit		;
;*********************************************************************************************************
;**********	Durchlauf hier alle 8 Sekunden ("Kopfintervall")				**********
;*********************************************************************************************************
;**********	Betriebsstunden zhlen								**********
;*********************************************************************************************************
	sbrs	flags,wiedergabe	;nur bei Aufnahme oder Wiedergabe
	rjmp	ende_betriebsstunden	;
	rcall	addiere_Kopfzeit	;Koptstunden zhlen
	rcall	rechne_stunden		;und einmal fr die Anzeige rechnen
ende_betriebsstunden:
	sts	sekunde,null		;Sekunden rcksetzen
;*********************************************************************************************************
ende_zeit:
	rjmp	start			;Ende Hauptschleife

;*********************************************************************************************************
;**********	Interruptroutine zum Auffrischen des Displays, CLK/8 = 2KHz Aufruffrequenz	**********
;**********	benutzte Register: var1[1:0], digit, Z, s, ticker, timer2, portc, portd		**********
;*********************************************************************************************************
t2ovf:
	in	s,sreg			;sichere Statusregister
	push	var10			;sichere Register
	push	var11			;
	push	zl			;
	push	zh			;
	ldi	var10,8			;Timer vorladen fr relativ genauen
	out	timer2,var10		;0,5ms-berlauf
	inc	ticker			;Ticker bedienen
;*********************************************************************************************************
;**********	Anzeigestelle rotieren								**********
;*********************************************************************************************************
	in	var10,portc		;sichere Portstatus
.if (gem_anode==1)			;fr Anzeigen mit gemeinsamer Kathode
	com	var10			;invertieren
.endif
	andi	var10,alle_digits	;Anzeigestellen maskieren
	cpi	var10,(1<<a_digit5)	;Multiplexrahmen am Ende?
	breq	erstes_digit		;dann von vorne
	lsl	var10			;sonst nchste Stelle ansteuern
	inc	digit			;Zeiger auf nchste Zelle anzeige
	rjmp	ende_digit		;
erstes_digit:
	clr	digit			;Zeiger auf erste Zelle anzeige

;*******  N E U  ***********************
	lds	var10,anzeige		;Ausgabespeicher aktualisieren
	sts	ausgabe,var10		;
	lds	var10,anzeige+1		;
	sts	ausgabe+1,var10		;
	lds	var10,anzeige+2		;
	sts	ausgabe+2,var10		;
	lds	var10,anzeige+3		;
	sts	ausgabe+3,var10		;
	lds	var10,anzeige+4		;
	sts	ausgabe+4,var10		;
;*******  N E U  ***********************

.if (anzeigestellen==4)			;bei 4 Digits
	inc	digit			;
	ldi	var10,(1<<a_digit2)	;erstes Digit bestimmen
.else					;bei 5 Digits
	ldi	var10,(1<<a_digit1)	;erstes Digit bestimmen
.endif
ende_digit:
	in	var11,portc		;sichere Portstatus
.if (gem_anode!=1)
	andi	var11,(1<<a_ir_led)	;lsche Anzeigestellen auer LED-Status
	or	var11,var10		;neue Anzeigestelle dazunehmen
.else
	com	var10			;invertieren
	ori	var11,~(1<<a_ir_led)	;lsche Anzeigestellen auer LED-Status
	and	var11,var10		;neue Anzeigestelle dazunehmen
	com	var10			;wieder Normallage, wird unten noch gebraucht
.endif
	out	portc,var11		;und Ausgabe
;*********************************************************************************************************
;**********	Helligkeit fr jede Anzeigestelle selektieren	  				**********
;*********************************************************************************************************
	lds	var11,blinkermaske	;Schablone fr Blinken laden
	andi	var11,alle_digits	;Digits maskieren
	and	var11,var10		;mit aktivem Digit vergleichen
	breq	helltasten		;soll Grundhelligkeit anliegen, dann hier lang, sonst
	lds	var11,blinkermaske	;nochmal Blinkschablone
	sbrs	var11,dimmer_ein	;zur Blinkfunktion
	rjmp	blinken			;hier lang, sonst
	lds	var10,dimmer		;Dimmfunktion
	rjmp	setze_helligkeit	;
blinken:
	lds	var10,blinker		;
	sbrs	var10,4			;alle 2^4 Frames
	rjmp	helltasten		;
	lds	var10,hell_absolut	;
	lsr	var10			;
	lsr	var10			;1/4 Grundhelligkeit laden
	rjmp	setze_helligkeit	;
helltasten:
	lds	var10,hell_absolut	;Grundelligkeit laden
setze_helligkeit:
	ldi	zh,byte2(tab_helligkeit<<1)	;Zeiger auf Tabelle Helligkeitsverteilung
	mov	zl,var10		;und Nutzdaten direkt adressieren
	lpm	var10,z			;Wert holen und
	out	match2,var10		;einstellen
;*********************************************************************************************************
;**********	Leuchtmuster fr diese Anzeigestelle holen					**********
;*********************************************************************************************************
	ldi	zl,byte1(ausgabe)	;Zeiger auf SRAM stellen
	ldi	zh,byte2(ausgabe)	;
	add	zl,digit		;passendes
	adc	zh,null			;
	ld	var10,z			;Leuchtbild holen
;*********************************************************************************************************
;**********	Ausgabe des Zeichens auf dem Display						**********
;*********************************************************************************************************
.if (gem_anode==1)			;fr Anzeigen mit gemeinsamer Anode
	com	var10			;Signal invertieren
.endif
	out	portd,var10		;Zeichen raus auf die Anzeige
	pop 	zh			;Register wiederherstellen
	pop	zl			;
	pop	var11			;
	pop	var10			;
	out	sreg,s			;Statusregister wiederherstellen
	reti

;*********************************************************************************************************
;**********	Interruptroutine zum Abschalten der Anzeige					**********
;**********	Hiermit wird die Displayhelligkeit eingestellt					**********
;**********	benutzte Register: portd, var10, s						**********
;*********************************************************************************************************
t2ocm:
	in	s,sreg			;sichere Statusregister
	push	var10			;sichere Register
	in	var10,portd		;
.if (gem_anode!=1)
	andi	var10,~alle_segmente	;alle Segmente
.else
	ori	var10,alle_segmente	;alle Segmente
.endif
	out	portd,var10		;abschalten
	pop	var10			;Register wiederherstellen
	out	sreg,s			;Statusregister wiederherstellen
	reti

;*********************************************************************************************************
;**********	Interruptroutine zur Messung der Zeit zwischen 2 Weifeldflanken		**********
;**********	benutzte Register: t, var1[1:0], icr, flags1					**********
;*********************************************************************************************************
t1cev:
	in	t,sreg			;Statusregister sichern
	push	var10			;Register sichern
	push	var11			;
	in	var10,icr1l		;atomarer Zugriff ntig?
	in	var11,icr1h		;neuen Zeitstempel sichern
	sts	icr,var10		;sichern
	sts	icr+1,var11		;
	sbr	flags1,(1<<segment_neu)	;und Bescheid sagen
	pop	var11			;Register wiederherstellen
	pop	var10			;
	out	sreg,t			;Statusregister wiederherstellen
	reti

;*********************************************************************************************************
;**********	Fehlervektoren									**********
;*********************************************************************************************************
fehler_isr_vektor:
	sbr	flags4,(1<<fehler_5)	;Fehlercode "5", falscher ISR-Vektor
	rjmp	reset_nach_fehler	;springe in die Initialisierung ohne Fehlerlschung
fehler_flashende:
	sbr	flags4,(1<<fehler_6)	;Fehlercode "6", falscher Lauf bis Flashende
	rjmp	reset_nach_fehler	;springe in die Initialisierung ohne Fehlerlschung
fehler_abschaltung:
	sbr	flags4,(1<<fehler_7)	;Fehlercode "7", falsches Sichern im EEPROM
	rjmp	reset_nach_fehler	;springe in die Initialisierung ohne Fehlerlschung

.include"unterprogramme.inc"
.include"anzeigeroutinen.inc"
.include"arithmetikroutinen.inc"
.include"daten.inc"


